/**
 * Created by mgoria on 5/28/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FS = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _s3fs = require('s3fs');

var _s3fs2 = _interopRequireDefault(_s3fs);

var _deepKernel = require('deep-kernel');

var _deepKernel2 = _interopRequireDefault(_deepKernel);

var _UnknownFolderException = require('./Exception/UnknownFolderException');

var _Exception = require('./Exception/Exception');

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _Registry = require('./Registry');

var _S3FsRumProxy = require('./S3FsRumProxy');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Deep FS implementation
 */
let FS = exports.FS = function (_Kernel$ContainerAwar) {
  _inherits(FS, _Kernel$ContainerAwar);

  /**
   * Defines all class private properties
   *
   * @param {String} tmpFsBucket
   * @param {String} publicFsBucket
   * @param {String} privateFsBucket
   * @param {String} sharedFsBucket
   */
  function FS() {
    let tmpFsBucket = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    let publicFsBucket = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    let privateFsBucket = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
    let sharedFsBucket = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

    _classCallCheck(this, FS);

    var _this = _possibleConstructorReturn(this, (FS.__proto__ || Object.getPrototypeOf(FS)).call(this));

    _this._mountedFolders = {};
    _this._buckets = {};

    _this._buckets[FS.TMP] = tmpFsBucket;
    _this._buckets[FS.PUBLIC] = publicFsBucket;
    _this._buckets[FS.PRIVATE] = privateFsBucket;
    _this._buckets[FS.SHARED] = sharedFsBucket;

    _this._sourcePath = null;
    _this._registry = null;
    return _this;
  }

  /**
   * @returns {Registry}
   * @constructor
   */


  _createClass(FS, [{
    key: 'boot',


    /**
     * Booting a certain service
     *
     * @param {Kernel} kernel
     * @param {Function} callback
     */
    value: function boot(kernel, callback) {
      let bucketsConfig = kernel.config.buckets;

      for (let i in FS.FOLDERS) {
        if (!FS.FOLDERS.hasOwnProperty(i)) {
          continue;
        }

        let folder = FS.FOLDERS[i];

        switch (folder) {
          case FS.TMP:
          case FS.PRIVATE:
            this._buckets[folder] = `${bucketsConfig[FS.PRIVATE].name}/${folder}/${kernel.microservice().identifier}`;
            break;
          case FS.SHARED:
            this._buckets[folder] = `${bucketsConfig[FS.PRIVATE].name}/${folder}`;
            break;
          case FS.PUBLIC:
            let publicBucketObj = bucketsConfig[folder];

            this._buckets[folder] = `${publicBucketObj.name}/${kernel.microservice().identifier}`;
            this._sourcePath = publicBucketObj.sourcePath;
            break;
          default:
            this._buckets[folder] = `${bucketsConfig[folder].name}/${kernel.microservice().identifier}`;
        }
      }

      callback();
    }

    /**
     * Returns mounted file private folder (tmp, public or private)
     *
     * @param {String} name
     * @param {String} msIdentifier
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */

  }, {
    key: 'getFolder',
    value: function getFolder(name) {
      let msIdentifier = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      if (FS.FOLDERS.indexOf(name) === -1) {
        throw new _UnknownFolderException.UnknownFolderException(name, FS.FOLDERS);
      }

      let realName = null;

      if (name === FS.SHARED) {
        if (!msIdentifier) {
          throw new _Exception.Exception('You must provide a microservice identifier for the shared fs');
        }

        // validate msIdentifier
        msIdentifier = this.kernel.microservice(msIdentifier).identifier;

        realName = `${name}.${msIdentifier}`;
      }

      if (typeof this._mountedFolders[realName || name] === 'undefined') {
        if (this._localBackend) {
          let rootFolder = FS._getTmpDir(this._buckets[name]);
          let SimulatedS3FS = require('./Local/S3FSRelativeFSExtender').S3FSRelativeFSExtender;

          let simulatedS3FS = new SimulatedS3FS(rootFolder); // relativeFsExtended

          if (name === FS.PUBLIC && this._sourcePath) {
            simulatedS3FS.addReadonlyDirectory(this._sourcePath);
          }

          this._mountedFolders[realName || name] = simulatedS3FS.relativeFsExtended;
        } else if (name === FS.SHARED) {
          let s3Fs = this._mountedFolders[name];

          if (typeof s3Fs === 'undefined') {
            s3Fs = new _s3fs2.default(this._buckets[name], {});

            if (this.kernel && this.kernel.isRumEnabled) {
              s3Fs = new _S3FsRumProxy.S3FsRumProxy(s3Fs, this.kernel.get('log')).proxy();
            }

            this._mountedFolders[name] = s3Fs;
          }

          this._mountedFolders[realName] = s3Fs.clone(msIdentifier);
        } else {
          let s3Fs = new _s3fs2.default(this._buckets[name], {});

          if (this.kernel && this.kernel.isRumEnabled) {
            s3Fs = new _S3FsRumProxy.S3FsRumProxy(s3Fs, this.kernel.get('log')).proxy();
          }

          this._mountedFolders[name] = s3Fs;
        }
      }

      return this._mountedFolders[realName || name];
    }

    /**
     * @param {String} subpath
     * @returns {String}
     * @private
     */

  }, {
    key: 'shared',


    /**
     * Returns mounted shared folder
     *
     * @param {Microservice|String|*} microservice
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */
    value: function shared() {
      let microservice = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

      microservice = microservice || this.kernel.microservice();

      let msIdentifier = typeof microservice === 'string' ? microservice : microservice.identifier;

      return this.getFolder(FS.SHARED, msIdentifier);
    }

    /**
     * Returns mounted tmp folder
     *
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */

  }, {
    key: 'registry',


    /**
     * @returns {Registry}
     */
    get: function get() {
      if (!this._registry) {
        this._registry = _Registry.Registry.createFromFS(this.private);
      }

      return this._registry;
    }

    /**
     * @returns {string}
     */

  }, {
    key: 'tmp',
    get: function get() {
      return this.getFolder(FS.TMP);
    }

    /**
     * Returns mounted public folder
     *
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */

  }, {
    key: 'public',
    get: function get() {
      return this.getFolder(FS.PUBLIC);
    }

    /**
     * Returns mounted system folder
     * @deprecated
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */

  }, {
    key: 'system',
    get: function get() {
      return this.getFolder(FS.PRIVATE);
    }

    /**
     * Returns mounted system folder
     *
     * @returns {fs|s3fs|S3FS|S3FsRumProxy|SimulatedS3FS|*}
     */

  }, {
    key: 'private',
    get: function get() {
      return this.getFolder(FS.PRIVATE);
    }
  }], [{
    key: '_getTmpDir',
    value: function _getTmpDir(subpath) {
      let dir = _path2.default.join(_os2.default.tmpdir(), subpath);

      require('fs-extra').mkdirpSync(dir);

      return dir;
    }
  }, {
    key: 'RegistryInstance',
    get: function get() {
      return _Registry.Registry;
    }
  }, {
    key: 'TMP',
    get: function get() {
      return 'temp';
    }

    /**
     * @returns {string}
     */

  }, {
    key: 'PUBLIC',
    get: function get() {
      return 'public';
    }

    /**
     * @returns {string}
     */

  }, {
    key: 'PRIVATE',
    get: function get() {
      return 'private';
    }

    /**
     * @returns {String}
     */

  }, {
    key: 'SHARED',
    get: function get() {
      return 'shared';
    }

    /**
     * @returns {Array}
     */

  }, {
    key: 'FOLDERS',
    get: function get() {
      return [FS.TMP, FS.PRIVATE, FS.SHARED, FS.PUBLIC];
    }
  }]);

  return FS;
}(_deepKernel2.default.ContainerAware);