/**
 * Created by AlexanderC on 10/27/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.S3FSRelativeFSExtender = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _relativeFs = require('relative-fs');

var _relativeFs2 = _interopRequireDefault(_relativeFs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('util');

var _es6Promise = require('es6-promise');

var _es6Promise2 = _interopRequireDefault(_es6Promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Fix missing Promise
_es6Promise2.default.polyfill();

let S3FSRelativeFSExtender = exports.S3FSRelativeFSExtender = function () {
  /**
   * @param {Object|String} relativeFsPath
   */
  function S3FSRelativeFSExtender(relativeFsPath) {
    _classCallCheck(this, S3FSRelativeFSExtender);

    let relativeFsObject = relativeFsPath;

    if (typeof relativeFsObject !== 'object') {
      relativeFsObject = _relativeFs2.default.relativeTo(relativeFsPath);
      relativeFsObject._rootFolder = relativeFsPath;
    } else if (!relativeFsObject.hasOwnProperty('_rootFolder')) {
      throw new Error('Missing _rootFolder property from RelativeFS object');
    }

    this._relativeFsObject = relativeFsObject;
  }

  /**
   * @returns {Object}
   */


  _createClass(S3FSRelativeFSExtender, [{
    key: 'addReadonlyDirectory',


    /**
     * @param {String} dir
     * @returns {S3FSRelativeFSExtender}
     */
    value: function addReadonlyDirectory(dir) {
      S3FSRelativeFSExtender.READ_METHODS.forEach(method => {
        let originalMethod = this._relativeFsObject[method];

        this._relativeFsObject[method] = function () {
          for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          let originalPath = args.shift();
          let originalCb = args.pop();

          originalMethod.apply(undefined, [originalPath].concat(args, [(error, data) => {
            if (error && error.code === 'ENOENT') {
              let absPath = _path2.default.join(dir, originalPath);

              return _fs2.default[method].apply(_fs2.default, [absPath].concat(args, [originalCb]));
            }

            originalCb(error, data);
          }]));
        };
      });

      return this;
    }

    /**
     * @returns {String[]}
     */

  }, {
    key: 'relativeFs',
    get: function get() {
      return this._relativeFsObject;
    }

    /**
     * @returns {Object}
     */

  }, {
    key: 'relativeFsExtended',
    get: function get() {
      return (0, _util._extend)(this._relativeFsObject, this.EXTEND_OBJECT);
    }

    /**
     * @returns {String}
     */

  }, {
    key: 'cwd',
    get: function get() {
      return _path2.default.normalize(this._relativeFsObject._rootFolder || process.cwd());
    }

    /**
     * @returns {{getPath: Function, clone: Function, copyFile: Function, create: Function, destroy: Function, headObject: Function, listContents: Function, readdirp: Function, mkdirp: Function, rmdirp: Function}}
     * @constructor
     */

  }, {
    key: 'EXTEND_OBJECT',
    get: function get() {
      var _this = this;

      let extendObject = {
        bucket: 'relative_fs',
        path: '',
        s3: {}, // @todo: mock `AWS.S3()`

        /**
         * @param {String} pathStr
         * @returns {String}
         *
         * @todo: return some fake bucket name? or the prefix?
         */
        getPath: function getPath() {
          let pathStr = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

          return _path2.default.join(_this.cwd, pathStr);
        },

        /**
         * @param {String} pathStr
         * @returns {Object}
         */
        clone: pathStr => {
          let newCwd = _path2.default.join(this.cwd, pathStr);

          return new S3FSRelativeFSExtender(newCwd).relativeFsExtended;
        },

        /**
         * @param {String} sourcePath
         * @param {String} destinationPath
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        copyFile: function copyFile(sourcePath, destinationPath) {
          let callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

          let absSourcePath = _path2.default.join(_this.cwd, sourcePath);
          let absDestinationPath = _path2.default.join(_this.cwd, destinationPath);

          if (callback) {
            _fsExtra2.default.copy(absSourcePath, absDestinationPath, callback);
            return;
          }

          return new Promise((resolve, reject) => {
            _fsExtra2.default.copy(absSourcePath, absDestinationPath, error => {
              if (error) {
                reject(error);
                return;
              }

              resolve();
            });
          });
        },

        /**
         * @param {Object} options
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        create: function create(options) {
          let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

          let error = new Error(`You do not have rights for this operation on bucket ${options.Bucket}`);

          if (callback) {
            callback(error);
            return;
          }

          return new Promise((resolve, reject) => {
            reject(error);
          });
        },

        /**
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        destroy: function destroy() {
          let callback = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

          let error = new Error('You do not have rights for this operation');

          if (callback) {
            _fsExtra2.default.remove(_this.cwd, () => {
              callback(error);
            });
            return;
          }

          return new Promise((resolve, reject) => {
            _fsExtra2.default.remove(_this.cwd, nativeError => {
              if (nativeError) {
                reject(nativeError);
                return;
              }

              reject(error);
            });
          });
        },

        /**
         * @param {String} pathStr
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        headObject: function headObject(pathStr) {
          let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

          let responseObj = {
            AcceptRanges: '',
            Restore: '',
            LastModified: new Date().getTime(),
            ContentLength: 0,
            ETag: '55ad340609f4b302',
            MissingMeta: 0,
            VersionId: '55ad340609f4b302',
            StorageClass: 'REDUCED_REDUNDANCY'
          };

          if (callback) {
            callback(responseObj);
            return;
          }

          return new Promise(resolve => {
            resolve(responseObj);
          });
        },

        /**
         * @param {String} pathStr
         * @param {String} marker
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        listContents: function listContents(pathStr, marker) {
          let callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

          let absPath = _path2.default.join(_this.cwd, pathStr);

          let globResponseObj = {
            Marker: marker,
            IsTruncated: false,
            Contents: [],
            Name: _this.cwd,
            Prefix: pathStr,
            Delimiter: '/',
            MaxKeys: 1000000000,
            CommonPrefixes: [],
            EncodingType: 'url'
          };

          let responseObj = {
            Key: null,
            LastModified: new Date().getTime(),
            Size: 0,
            ETag: '55ad340609f4b302',
            StorageClass: 'REDUCED_REDUNDANCY',
            Owner: {
              DisplayName: 's3fs',
              ID: '55ad340609f4b302'
            }
          };

          if (callback) {
            _fsExtra2.default.walk(absPath).on('data', item => {
              globResponseObj.Contents.push((0, _util._extend)(responseObj, { Key: item.path.substr(_this.cwd.length) }));
            }).on('end', () => {
              callback(globResponseObj);
            });

            return;
          }

          return new Promise(resolve => {
            _fsExtra2.default.walk(absPath).on('data', item => {
              globResponseObj.Contents.push((0, _util._extend)(responseObj, { Key: item.path.substr(_this.cwd.length) }));
            }).on('end', () => {
              resolve(globResponseObj);
            });
          });
        },

        /**
         * @param {String} pathStr
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        readdirp: function readdirp(pathStr) {
          let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

          let absPath = _path2.default.join(_this.cwd, pathStr);

          if (callback) {
            try {
              callback(null, S3FSRelativeFSExtender._readdirp(absPath, _this.cwd));
            } catch (error) {
              callback(error, null);
            }

            return;
          }

          return new Promise((resolve, reject) => {
            try {
              resolve(S3FSRelativeFSExtender._readdirp(absPath, _this.cwd));
            } catch (error) {
              reject(error);
            }
          });
        },

        /**
         * @param {String} pathStr
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        mkdirp: function mkdirp(pathStr) {
          let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

          let absPath = _path2.default.join(_this.cwd, pathStr);

          if (callback) {
            _fsExtra2.default.mkdirp(absPath, callback);
            return;
          }

          return new Promise((resolve, reject) => {
            _fsExtra2.default.mkdirp(absPath, error => {
              if (error) {
                reject(error);
                return;
              }

              resolve();
            });
          });
        },

        /**
         * @param {String} pathStr
         * @param {Function|null} callback
         * @returns {Promise|undefined}
         */
        rmdirp: function rmdirp(pathStr) {
          let callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

          let absPath = _path2.default.join(_this.cwd, pathStr);

          if (callback) {
            _fsExtra2.default.remove(absPath, callback);
            return;
          }

          return new Promise((resolve, reject) => {
            _fsExtra2.default.remove(absPath, error => {
              if (error) {
                reject(error);
                return;
              }

              resolve();
            });
          });
        },

        /**
         * @param {String} pathStr
         * @param {String} content
         * @param {Function} callback
         * @returns {Promise|undefined}
         */
        writeFile: (pathStr, content, callback) => {
          let absPath = _path2.default.join(this.cwd, pathStr);

          if (callback) {
            _fsExtra2.default.outputFile(absPath, content, callback);
            return;
          }

          return new Promise((resolve, reject) => {
            _fsExtra2.default.outputFile(absPath, content, error => {
              if (error) {
                reject(error);
                return;
              }

              resolve();
            });
          });
        }
      };

      extendObject.copyDir = extendObject.copyFile;
      extendObject.delete = extendObject.create;
      extendObject.putBucketLifecycle = extendObject.create;

      return extendObject;
    }

    /**
     * @param {String} dir
     * @param {String} basePath
     * @returns {String[]}
     * @private
     */

  }], [{
    key: '_readdirp',
    value: function _readdirp(dir) {
      let basePath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      let _files = [];

      let files = _fs2.default.readdirSync(dir);

      for (let i in files) {
        if (!files.hasOwnProperty(i)) {
          continue;
        }

        let file = _path2.default.join(dir, files[i]);

        _files.push(file);

        if (_fs2.default.statSync(file).isDirectory()) {
          _files = _files.concat(S3FSRelativeFSExtender._readdirp(file));
        }
      }

      return basePath ? _files.map(file => file.substr(basePath.length)) : _files;
    }
  }, {
    key: 'READ_METHODS',
    get: function get() {
      return ['readFile'];
    }
  }]);

  return S3FSRelativeFSExtender;
}();