/**
 * Created by AlexanderC on 1/21/16.
 */

//need for backward compatibility with node v0.10.x
/*eslint no-eq-null: 0, eqeqeq: 0*/

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Registry = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

let Registry = exports.Registry = function () {
  /**
   * @param {AWS.S3} s3
   * @param {String} bucket
   * @param {String} registryFile
   */
  function Registry(s3, bucket) {
    let registryFile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Registry.REGISTRY_FILE;

    _classCallCheck(this, Registry);

    this._s3 = s3;
    this._bucket = bucket;
    this._registryFile = registryFile;

    this._registry = null;
    this._lastModified = null;
    this._ensureFresh = Registry.ENSURE_FRESH;
  }

  /**
   * @param {FS|*} fs
   * @param {String} registryFile
   * @returns {Registry}
   */


  _createClass(Registry, [{
    key: 'keys',


    /**
     * @param {Function} cb
     * @returns {Registry}
     */
    value: function keys(cb) {
      this._ensureLoaded(error => {
        if (error) {
          cb(error, null);
          return;
        }

        cb(null, Object.keys(this._registry));
      });

      return this;
    }

    /**
     * @param {String} key
     * @param {Function} cb
     * @returns {Registry}
     */

  }, {
    key: 'read',
    value: function read(key, cb) {
      this._ensureLoaded(error => {
        if (error) {
          cb(error, null);
          return;
        }

        if (this._registry.hasOwnProperty(key)) {
          cb(null, this._registry[key]);
          return;
        }

        cb(new Error(`Missing registry entry '${key}'`), null);
      });

      return this;
    }

    /**
     * @param {String} key
     * @param {Function} cb
     * @returns {Registry}
     */

  }, {
    key: 'exists',
    value: function exists(key, cb) {
      this._ensureLoaded(error => {
        if (error) {
          cb(error, null);
          return;
        }

        cb(null, this._registry.hasOwnProperty(key));
      });

      return this;
    }

    /**
     * @param {String} key
     * @param {*} data
     * @param {Function} cb
     * @returns {Registry}
     */

  }, {
    key: 'write',
    value: function write(key, data, cb) {
      this._ensureLoaded(error => {
        if (error) {
          cb(error);
          return;
        }

        let hadKey = this._registry.hasOwnProperty(key);
        let oldValue = this._registry[key];

        this._registry[key] = data;

        this._persist(error => {
          if (error) {
            if (hadKey) {
              this._registry[key] = oldValue;
            } else {
              delete this._registry[key];
            }
          }

          cb(error);
        });
      });

      return this;
    }

    /**
     * @param {Function} cb
     * @returns {Registry}
     */

  }, {
    key: 'refresh',
    value: function refresh(cb) {
      this._readS3Object(this._registryFile, (error, rawRegistry, lastModified) => {
        if (error && error.code !== 'NoSuchKey') {
          cb(error);
          return;
        }

        if (rawRegistry) {
          try {
            this._registry = Registry._decodeRegistry(rawRegistry);
            this._lastModified = lastModified;
          } catch (error) {
            cb(error);
            return;
          }
        }

        cb(null);
      }, this._lastModified);

      return this;
    }

    /**
     * @param {Function} cb
     * @private
     */

  }, {
    key: '_persist',
    value: function _persist(cb) {
      this._writeS3Object(this._registryFile, Registry._encodeRegistry(this._registry), cb);
    }

    /**
     * @param {Function} cb
     * @private
     */

  }, {
    key: '_ensureLoaded',
    value: function _ensureLoaded(cb) {
      if (this._registry === null || this._ensureFresh) {
        this.refresh(error => {
          this._registry = this._registry || {};

          cb(error);
        });
      } else {
        cb(null);
      }
    }

    /**
     * @param {String} rawRegistry
     * @returns {Object}
     * @private
     */

  }, {
    key: '_readS3Object',


    /**
     * @param {String} key
     * @param {Function} cb
     * @param {Date|null} lastModified
     * @private
     */
    value: function _readS3Object(key, cb) {
      let lastModified = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      let payload = {
        Bucket: this._bucket,
        Key: key
      };

      if (lastModified) {
        payload.IfModifiedSince = lastModified;
      }

      this._s3.getObject(payload, (error, data) => {
        if (error) {

          // when the object is not modified
          if (lastModified && error.code === 'NotModified') {
            cb(null, null, null);
            return;
          }

          cb(error, null, null);
          return;
        }

        let result = null;
        let resultLastModified = lastModified;

        if (data && data.Body) {
          result = data.Body.toString();

          resultLastModified = typeof data.LastModified === 'string' ? new Date(data.LastModified) : data.LastModified;
        }

        cb(null, result, resultLastModified);
      });
    }

    /**
     * @param {String} key
     * @param {*} content
     * @param {Function} cb
     * @private
     */

  }, {
    key: '_writeS3Object',
    value: function _writeS3Object(key, content, cb) {
      let payload = {
        Bucket: this._bucket,
        Key: key,
        Body: content.toString()
      };

      this._s3.putObject(payload, error => {
        cb(error);
      });
    }

    /**
     * @param {*} value
     * @returns {{t: string, v: *}}
     * @private
     */

  }, {
    key: 'ensureFresh',


    /**
     * @param {Boolean} state
     */
    set: function set(state) {
      this._ensureFresh = state;
    }

    /**
     * @returns {Boolean}
     */
    ,
    get: function get() {
      return this._ensureFresh;
    }

    /**
     * @returns {Date}
     */

  }, {
    key: 'lastModified',
    get: function get() {
      return this._lastModified;
    }

    /**
     * @returns {String}
     */

  }, {
    key: 'registryFile',
    get: function get() {
      return this._registryFile;
    }

    /**
     * @returns {String}
     */

  }, {
    key: 'bucket',
    get: function get() {
      return this._bucket;
    }

    /**
     * @returns {AWS.S3}
     */

  }, {
    key: 's3',
    get: function get() {
      return this._s3;
    }

    /**
     * @returns {Boolean}
     */

  }], [{
    key: 'createFromFS',
    value: function createFromFS(fs) {
      let registryFile = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Registry.REGISTRY_FILE;

      return new Registry(fs.s3, fs.bucket, fs.getPath(registryFile));
    }
  }, {
    key: '_decodeRegistry',
    value: function _decodeRegistry(rawRegistry) {
      let registry = {};
      let registryObj = JSON.parse(rawRegistry);

      for (let key in registryObj) {
        if (!registryObj.hasOwnProperty(key)) {
          continue;
        }

        registry[key] = Registry._decodeValue(registryObj[key]);
      }

      return registry;
    }

    /**
     * @param {Object} registry
     * @returns {String}
     * @private
     */

  }, {
    key: '_encodeRegistry',
    value: function _encodeRegistry(registry) {
      let registryObj = {};

      for (let key in registry) {
        if (!registry.hasOwnProperty(key)) {
          continue;
        }

        registryObj[key] = Registry._encodeValue(registry[key]);
      }

      return JSON.stringify(registryObj);
    }
  }, {
    key: '_encodeValue',
    value: function _encodeValue(value) {
      let type = 'plain';
      let val = value;

      if (_util2.default.isArray(value)) {
        type = 'arr';
        val = JSON.stringify(value);
      } else if (Buffer.isBuffer(value)) {
        val = value.toString();
      } else if (Registry.isObject(value)) {
        type = 'obj';
        val = JSON.stringify(value);
      }

      return {
        t: type,
        v: val
      };
    }

    /**
     * @param {*} obj
     * @returns {boolean}
     */

  }, {
    key: 'isObject',
    value: function isObject(obj) {
      return obj != null && typeof obj === 'object' && !Array.isArray(obj);
    }

    /**
     * @param {Object} valueObj
     * @returns {*}
     * @private
     */

  }, {
    key: '_decodeValue',
    value: function _decodeValue(valueObj) {
      let value = null;

      switch (valueObj.t) {
        case 'arr':
        case 'obj':
          value = JSON.parse(valueObj.v);
          break;
        default:
          value = valueObj.v;
      }

      return value;
    }
  }, {
    key: 'ENSURE_FRESH',
    get: function get() {
      return true;
    }

    /**
     * @returns {String}
     */

  }, {
    key: 'REGISTRY_FILE',
    get: function get() {
      return '__deep_registry__';
    }
  }]);

  return Registry;
}();